from fastflows.providers.prefect import PrefectProvider

# it's here because it is default right now, if will be supported more providers:
# add logic to get provider from configuration
provider = PrefectProvider()
